import createComponent from '../../../dist/redom.js';
import listItem from './item.js';

export default createComponent({
  create(tasks) {
    this.model = {
      tasks
    };
  },
  view() {
    return (
      <section class="main">
        <input class="toggle-all" type="checkbox"></input>
        <label for="toggle-all">Mark all as complete</label>
        <ul ref="list" class="todo-list"></ul>
    </section>
    );
  },
  render(container) {
    const node = this.view();
    container.appendChild(node);
    return node;
  },
  addTask(task) {
    const item = listItem(task);
    task._item = item;
    this.model.tasks.push(task);
    item.render(this.refs.list);
    item.on('removed', this.removeItem);
    item.on('updated', this._updated);
    if (typeof this.model.filter === 'function') {
      item.display(this.model.filter(item));
    }
    this.trigger('updated');
  },
  filter(p) {
    this.model.filter = p;
    this.model.tasks.forEach((it) => {
      it._item.display(p(it));
    });
  },
  clearCompleted() {
    const tasks = this.model.tasks;
    for (let it = tasks.length - 1; it >= 0; it--) {
      if (tasks[it].completed) {
        const item = this.model.tasks.splice(it, 1)[0];
        item._item.detach().destroy();
        item._item = null;
      }
    }
  }
}, {
  // Events
  removeItem(id) {
    const idx = this.model.tasks.findIndex(i => i.id === id);
    if (idx !== -1) {
      const item = this.model.tasks.splice(idx, 1)[0];
      item._item.detach();
      item._item = null;
      this.trigger('updated');
    }
  },
  _updated(item) {
    item.display(this.model.filter(item));
    this.trigger('updated');
  }
});
